/*
    ES Knot
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __KNOTAPPVIEW_H
#define __KNOTAPPVIEW_H

#include <coecntrl.h>
#include <gles/egl.h>

class Knot;

class CKnotAppView : public CCoeControl, public MDirectScreenAccess {
public:

	enum TDrawMode { EEGLWindow, EDoubleBuf, EDSA };
	enum TWindowMode { EWindowed, EFullscreen };

	static CKnotAppView* NewL(const TRect& aRect, TBool aFullscreen);
	static CKnotAppView* NewLC(const TRect& aRect, TBool aFullscreen);
	~CKnotAppView();
	void Draw(const TRect& aRect) const;
	static int StaticCallback(TAny* aArg);

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

	void SetForeground(TBool aForeground);

	void Restart(RDirectScreenAccess::TTerminationReasons aReason);
	void AbortNow(RDirectScreenAccess::TTerminationReasons aReason);

	void SetDrawMode(TDrawMode aMode, TBool aReset = EFalse);
	void SetWindowMode(TWindowMode aMode);

private:
	void ConstructL(const TRect& aRect, TBool aFullscreen);
	CKnotAppView();

	void SetupBitmap();
	void Update();
	void InitEGL(TBool aPixmap);
	void Callback();


	EGLDisplay iDisplay;
	EGLContext iContext;
	EGLSurface iSurface;
	CPeriodic* iPeriodic;

	CFbsBitmap* iBitmap;
	CFbsBitmapDevice* iDevice;
	CFbsBitGc* iGc;
	const CFont* iFont;
	CDirectScreenAccess* iDrawer;
	TBool iDSAStarted;

	TInt iFrame;
	TInt iPeriod;
	TUint iLastFPSTime;
	TInt iLastFPSFrame;
	TInt iLastFPS;
	TBool iStarted;
	TInt iLastTick;
	TUint iTime;
	TInt iFrameTickLength;
	TInt *iFrameTick;
	TInt iRelTick;
	TInt iLastResetTick;
	TInt iLastAfterTick;

	TDrawMode iDrawMode;
	TWindowMode iWinMode;
	TBool iEGLInited;

	TRect iOrigRect;

public:
	Knot* iKnot;
};

#endif
